<?php
/*
Plugin Name: Pizazz
Plugin URI: http://entertainingsoftware.com/wordpress/pizazz/
Description: Zazzle feed widget
Version: 1.0.0
Author: Luiji Maryo
Author URI: http://profiles.wordpress.org/luiji
License: GPLv2 or later
Text Domain: pizazz
Domain Path: /languages/
*/

/*
Pizazz: Zazzle feed widget for WordPress
Copyright (C) 2012 Entertaining Software, Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

/**
 * @package   Pizazz
 * @author    Luiji <luiji@users.sourceforge.net>
 * @copyright 2012 Entertaining Software, Inc.
 * @license   http://www.gnu.org/licenses/gpl-2.0.html GPLv2+
 *
 * Pizazz Zazzle feed plugin
 */

// TODO : update URLs to correct values (in readme.txt, too)

// prevent people from trying to access this file directly
if ( ! function_exists( 'add_action' ) )
	echo "'Ello, 'ello, what's all this then? This is a WordPress plug-in, you can't call it directly.";

// load up any translation data
// note: currently pizazz is maintained outside of a directory; if
// translations are added, pizazz should be moved into a directory
load_plugin_textdomain( 'pizazz', false, basename( dirname( __FILE__ ) ) . '/languages' );

/**
 * Zazzle product parser and storage class
 */
class Pizazz_Product extends WP_Widget {

	/**
	 * Product title (defaults to "(Plugin Error)")
	 */
	public $title = '(Plugin Error)';

	/**
	 * Product link (defaults to example.com)
	 */
	public $link = 'http://example.com';

	/**
	 * Thumbnail URI (defaults to Zazzle's 1x1 blank image)
	 */
	public $thumbnail = 'http://asset.zcache.com/assets/graphics/design/err/blank.gif';

	/**
	 * Constructor. Initializes the product with a SimpleXML node.
	 *
	 * @param SimpleXMLElement $node RSS <item> node
	 */
	function __construct( $node ) {
		// extract plain attributes
		$this->title = $node->title;
		$this->link = $node->link;

		// extract the thumbnail (this weird loop is the best I could manage with SimpleXML)
		$this->thumbnail = '';
		foreach ( $node->xpath( './media:thumbnail' ) as $thumbnail ) {
			$this->thumbnail = $thumbnail['url'];
		}
	}

	/**
	 * Resize the thumbnail by adjusting its URI. Note that the width
	 * and height are always equal, so only one dimension is given.
	 *
	 * @param int $thumbsize Size of the thumbnail
	 */
	function resize_thumbnail( $thumbsize ) {
		if ( 0 == $thumbsize ) {
			// do not use a thumbnail
			$this->thumbnail = '';
		}
		else {
			// the digits after the last underscore indicate the size (width always equals height)
			$this->thumbnail = str_replace( '_125.jpg', "_$thumbsize.jpg", $this->thumbnail );
		}
	}

	/**
	 * Set associate ID for referrals by adjusting product link. This is
	 * a very important feature: whoever gets that referral earns 15% of
	 * all sales made through the link.
	 *
	 * @param string $associate Zazzle 18-digit associate ID
	 */
	function set_associate( $associate ) {
		// append rf=$associate, using a ? if it is the lone parameter and & if there is/are other(s)
		if ( strpos ( $this->link, '?' ) ) // there are other parameters
			$this->link .= "&rf=$associate";
		else // this is the lone parameter
			$this->link .= "?rf=$associate";
	}

	/**
	 * Get the HTML equivalent of the product.
	 *
	 * @return String with the HTML equivalent of the product
	 */
	function to_html() {
		$result = '';

		// localize properties for simpler echo commands
		$title = $this->title;
		$link = $this->link;
		$thumb = $this->thumbnail;

		// output the thumbnail if one was wanted
		if ( $this->thumbnail != '' )
			$result .= "<div><a href='$link'><img src='$thumb' alt='$title' /></a></div>";

		// output the textual link
		$result .= "<div><a href='$link'>$title</a></div>";

		return $result;
	}

}; // class Pizazz_Product

/**
 * Pizazz widget class
 */
class Pizazz_Widget extends WP_Widget {

	/**
	 * Constructor. Initializes the widget and finalizes its
	 * registration with WordPress.
	 */
	function __construct() {
		parent::__construct(
			'pizazz_widget',
			__( 'Pizazz Widget', 'pizazz' ),
			array( 'description' => __( 'Zazzle feed widget', 'pizazz' ) )
		);
	}

	/**
	 * Output the options form for administrator.
	 *
	 * @param array Values associated with the current widget instance
	 */
	function form( $instance ) {
		// each property block consists of three tasks:
		// 1. get the current value of the property or its default value
		// 2. get the property's id, name and translated label
		// 3. output the HTML

		// *** static information *** //
		echo '<p><em>All fields below are optional.</em></p>';

		// *** property: title *** //

		if ( isset ( $instance['title'] ) )
			$title = esc_attr( $instance['title'] );
		else
			$title = __( 'Zazzle Products', 'pizazz' );

		$title_id = $this->get_field_id( 'title' );
		$title_name = $this->get_field_name( 'title' );
		$title_label = __( 'Title:', 'pizazz' );

		echo '<p>';
		echo "<label for='$title_id'>$title_label</label>";
		echo "<input class='widefat' id='$title_id' name='$title_name' type='text' value='$title' />";
		echo '</p>';

		// *** property: store *** //

		if ( isset ( $instance['store'] ) )
			$store = esc_attr( $instance['store'] );
		else
			$store = '';

		$store_id = $this->get_field_id( 'store' );
		$store_name = $this->get_field_name( 'store' );
		$store_label = __( 'Store:', 'pizazz' );

		echo '<p>';
		echo "<label for='$store_id'>$store_label</label>";
		echo "<input class='widefat' id='$store_id' name='$store_name' type='text' value='$store' />";
		echo '</p>';

		// *** property: keywords *** //

		if ( isset ( $instance['keywords'] ) )
			$keywords = esc_attr( $instance['keywords'] );
		else
			$keywords = '';

		$keywords_id = $this->get_field_id( 'keywords' );
		$keywords_name = $this->get_field_name( 'keywords' );
		$keywords_label = __( 'Keywords (i.e. "christmas shirt"):', 'pizazz' );

		echo '<p>';
		echo "<label for='$keywords_id'>$keywords_label</label>";
		echo "<input class='widefat' id='$keywords_id' name='$keywords_name' type='text' value='$keywords' />";
		echo '</p>';

		// *** property: associate *** //

		if ( isset ( $instance['associate'] ) )
			$associate = esc_attr( $instance['associate'] );
		else
			$associate = '';

		$associate_id = $this->get_field_id( 'associate' );
		$associate_name = $this->get_field_name( 'associate' );
		$associate_label = __( 'Associate ID:', 'pizazz' );

		echo '<p>';
		echo "<label for='$associate_id'>$associate_label</label>";
		echo "<input class='widefat' id='$associate_id' name='$associate_name' type='text' value='$associate' />";
		echo '</p>';

		// *** property: thumbnail size *** //

		if ( isset ( $instance['thumbsize'] ) )
			$thumbsize = intval( $instance['thumbsize'] );
		else
			$thumbsize = 125;

		$thumbsize_id = $this->get_field_id( 'thumbsize' );
		$thumbsize_name = $this->get_field_name( 'thumbsize' );
		$thumbsize_label = __( 'Thumbnail Size (set to 0 if you do not want one):', 'pizazz' );

		echo '<p>';
		echo "<label for='$thumbsize_id'>$thumbsize_label</label>";
		echo "<input class='widefat' id='$thumbsize_id' name='$thumbsize_name' type='text' value='$thumbsize' />";
		echo '</p>';

		// *** property: thumbnail background color *** //
		if ( isset ( $instance['thumbback'] ) )
			$thumbback = $instance['thumbback'];
		else
			$thumbback = 'ffffff';

		$thumbback_id = $this->get_field_id( 'thumbback' );
		$thumbback_name = $this->get_field_name( 'thumbback' );
		$thumbback_label = __( 'Thumbnail Color (<a href="http://en.wikipedia.org/wiki/Hex_colors#Hex_triplet">6-digit hex format</a> without "#")', 'pizazz' );

		echo '<p>';
		echo "<label for='$thumbback_id'>$thumbback_label</label>";
		echo "<input class='widefat' id='$thumbback_id' name='$thumbback_name' type='text' value='$thumbback' />";
		echo '</p>';

		// *** property: column count *** //

		if ( isset ( $instance['columns'] ) )
			$columns = intval( $instance['columns'] );
		else
			$columns = 1;

		$columns_id = $this->get_field_id( 'columns' );
		$columns_name = $this->get_field_name( 'columns' );
		$columns_label = __( 'Column Count (must be at least 1):', 'pizazz' );

		echo '<p>';
		echo "<label for='$columns_id'>$columns_label</label>";
		echo "<input class='widefat' id='$columns_id' name='$columns_name' type='text' value='$columns' />";
		echo '</p>';

		// *** property: row count *** //

		if ( isset ( $instance['rows'] ) )
			$rows = intval( $instance['rows'] );
		else
			$rows = 3;

		$rows_id = $this->get_field_id( 'rows' );
		$rows_name = $this->get_field_name( 'rows' );
		$rows_label = __( 'Row Count (must be at least 1):', 'pizazz' );

		echo '<p>';
		echo "<label for='$rows_id'>$rows_label</label>";
		echo "<input class='widefat' id='$rows_id' name='$rows_name' type='text' value='$rows' />";
		echo '</p>';
	} // function form

	/**
	 * Generate an array representing the widget's saved state.
	 *
	 * @param array $new_instance Data from the new widget instance
	 * @param array $old_instance Data from the old widget instance
	 * @return Array of values associated with the saved widget
	 */
	function update( $new_instance, $old_instance ) {
		return array(
			'title' => strip_tags( $new_instance['title'] ),
			'store' => strip_tags( $new_instance['store'] ),
			'keywords' => strip_tags( $new_instance['keywords'] ),
			'associate' => strip_tags( $new_instance['associate'] ),
			'thumbsize' => intval( $new_instance['thumbsize'] ),
			'thumbback' => strip_tags( $new_instance['thumbback'] ),
			'columns' => intval( $new_instance['columns'] ),
			'rows' => intval( $new_instance['rows'] )
		);
	}

	/**
	 * Output the widget to the WordPress website.
	 *
	 * @param array $args     Arguments effecting the widget's output
	 * @param array @instance Values associated with the widget
	 */
	function widget( $args, $instance ) {
		// get and parse widget properties
		$title = esc_html( $instance['title'] );
		$store = urlencode( $instance['store'] );
		$keywords = urlencode( $instance['keywords'] );
		$associate = urlencode( $instance['associate'] );
		$thumbsize = intval( $instance['thumbsize'] );
		$thumbback = urlencode( $instance['thumbback'] );
		$columns = intval( $instance['columns'] );
		$rows = intval( $instance['rows'] );

		// begin the widget
		echo $args['before_widget'];

		// echo the title, if any
		if ( ! empty( $instance['title'] ) ) {
			echo $args['before_title'];
			echo esc_html( $instance['title'] );
			echo $args['after_title'];
		}

		// download the Zazzle RSS feed
		$rssurl = "http://feed.zazzle.com/$store/rss?qs=$keywords&bg=$thumbback";
		$rssget = wp_remote_get( $rssurl, array( 'redirection' => false ) );

		// ensure the get was successful
		if ( is_wp_error ( $rssget ) ) {
			// get error message
			$error = $rssget->get_error_message();

			// if it was redirection, then the store does not exist; otherwise, it was an HTTP error
			if ( strpos ( $error, 'redirect' ) )
				echo "<p><strong>Error: the store '$store' cannot be found. Either it does not exist or Zazzle was inaccessible.";
			else
				echo '<p><strong>Error: failed to access Zazzle. ', $error, '</strong></p>';

			// skip the products
			goto skip_products;
		}

		// parse the Zazzle RSS feed
		$rssxml = simplexml_load_string( $rssget['body'] );
		$rssxml->registerXPathNamespace( 'media', 'http://search.yahoo.com/mrss/' );

		// process products
		$products = array();
		$count = 0;
		foreach ( $rssxml->channel->item as $product_node ) {
			// process the node information
			$product = new Pizazz_Product( $product_node );

			// configure the thumbnail size
			$product->resize_thumbnail( $thumbsize );

			// add the associate referral id if there was one
			if ( ! empty ( $associate ) )
				$product->set_associate( $associate );

			// append the product to the list
			$products[] = $product;

			// increment the product count
			++$count;
		}

		// list products if the query was successful, display an error otherwise
		if ( ! $count ) {
			// notify the user that no results were found
			// $instance[] is used when escaped/encoded strings are not wanted
			echo '<strong>';
			echo '<p>Error: zero results found using the specified parameters.</p>';
			echo "<p>Store: <a href='http://zazzle.com/$store?rf=$associate'>", $instance['store'], '</a></p>';
			echo '<p>Keywords: ', $instance['keywords'], '</p>';
			echo '</strong>';

			// skip the products
			goto skip_products;
		}

		// shorten the product count if there are more than the available cells
		// (this mechanism is used for the less likely condition that there are too few products, in which case array_rand() fails)
		if ( $count >= $rows * $columns )
			$count = $rows * $columns;

		// generate unique random numbers for products
		$wanted_products = array_rand( $products, $count );

		// $i is used to ensure we do not exceed $product_count
		$i = 0;

		// begin widget table
		echo '<table style="width: 100%;">';

		// iterate through the rows
		for ( $row = 0; $row < $rows and $i < $count; ++$row ) {
			echo '<tr>';

			// iterate through the columns
			for ( $column = 0; $column < $columns and $i < $count; ++$column ) {
				echo '<td style="vertical-align: top; text-align: center;">';
				echo $products[$wanted_products[$i++]]->to_html();
				echo '</td>';
			}

			echo '</tr>';
		}

		// end widget table
		echo '</table>';

		// this label is used during errors
	skip_products:

		// output a "see more" link
		echo "<div style='text-align: center;'><em><a href='http://zazzle.com/$store/$keywords+gifts?rf=$associate'>", __( 'See More', 'pizazz' ), '</a></em></div>';

		// end the widget
		echo $args['after_widget'];
	} // function widget

} // class Pizazz_Widget

/**
 * Register the Pizazz widget with WordPress.
 */
function pizazz_register_widget() {
	register_widget( 'Pizazz_Widget' );
}

// queue the widget to be registered _after_ the widget subsystem is initialized
add_action( 'widgets_init', 'pizazz_register_widget' );

// *** end ***
